
def get_validate_hanzi():
    validate_hanzi = file("valid_utf16.txt").read().decode("utf16")
    return set(validate_hanzi)

def main():
    hanzi = get_validate_hanzi()
    hanzi = list(hanzi)
    hanzi.sort()
    print "# -*- coding: utf-8 -*- "
    print "valid_hanzi = set(["
    for c in hanzi:
        print "    u\"%s\"," % c.encode("utf8")
    print "])"

if __name__ == "__main__":
    main()



import sctc
items = sctc.S_2_T.items()
items.sort()
print "const gunichar simp_to_trad[][2] = {"
for s, t in items:
    print "    { 0x%x, 0x%x },   // %s => %s" % (ord(s), ord(t[0]), s.encode("utf8"), t[0].encode("utf8"))
print "};"

print '#define SIMP_TO_TRAD_NR (sizeof (simp_to_trad) / sizeof (simp_to_trad[0]))'
print




#!/usr/bin/env python
import sys
sys.path.append(".")

from ZhConversion import *
from valid_hanzi import *

def convert(s, d, n):
    out = ""
    end = len(s)
    begin = 0
    while begin < end:
        for i in range(min(n, end - begin), 0, -1):
            t = s[begin:begin+i]
            t = d.get(t, t if i == 1 else None)
            if t:
                break
        out = out + t
        begin += i
    return out

def filter_more(records, n):
    han = [(k, v) for (k, v) in records if len(k) <= 0]
    hand = dict(han)
    hanm = [(k, v) for (k, v) in records if convert(k, hand, n) != v]
    return hanm + han

def filter_func(args):
    k, v = args
    # length is not equal or length > 6
    if len(k) != len(v) or len(k) > 6:
        return False
    # k includes invalid hanzi
    if not all(c in valid_hanzi for c in k):
        return False
    # v includes invalid hanzi
    if not all(c in valid_hanzi for c in v):
        return False

    # # check chars in k and v
    # for c1, c2 in zip(k, v):
    #     if c1 == c2:
    #         continue
    #     if c2 not in S_2_T.get(c1, []):
    #         return False
    return True

def get_records():
    records = list(zh2Hant.items())

    records = list(filter(filter_func, records))

    maxlen = max([len(k) for (k, v) in records])
    for i in range(1,  maxlen - 1):
        records = filter_more(records, i)
    records = [(k.encode("utf8"), v.encode("utf8")) for (k, v) in records]
    records.sort()
    return maxlen, records

def main():
    print("static const gchar *simp_to_trad[][2] = {")
    maxlen, records = get_records()
    for s, ts in records:
        print('    { "%s", "%s" },' % (s, ts))
    print("};")
    print('#define SIMP_TO_TRAD_MAX_LEN (%d)' % maxlen)

if __name__ == "__main__":
    main()



#!/usr/bin/python3
import os
import operator
from argparse import ArgumentParser

import xml.etree.ElementTree as ET

header = '''/* This file is generated by python scripts. Don't edit this file directly.
 */
'''

alphabet = "abcdefghijklmnopqrstuvwxyz"

eng_emojis = []
chs_emojis = []

def load_emoji(filename):
    tree = ET.parse(filename)
    root = tree.getroot()

    emojis = {}
    for annotation in root.findall('.//annotation'):
        for word in annotation.text.split('|'):
            word = word.strip()

            # only keep the first encountered emoji
            if not word in emojis:
                # print(annotation.get('cp'))
                emojis[word] = annotation.get('cp')

    return emojis

# no space allowed for English emoji

def filter_English_emoji(emojis):
    emojis_copy = {}

    for key, value in emojis.items():
        if ' ' in key:
            continue

        if len(key) > 6:
            continue

        # only accept alphabet
        isalphabet = True
        for c in key:
            if not c in alphabet:
                isalphabet = False

        if not isalphabet:
            continue

        #print(key, value)
        emojis_copy[key] = value

    return emojis_copy


# less than four characters for Chinese emoji

def filter_Chinese_emoji(emojis):
    emojis_copy = {}

    for key, value in emojis.items():
        if len(key) > 3:
            continue

        # just reject alphabet
        isalnum = False
        for c in key:
            if c.isdigit() or c in alphabet or c in alphabet.upper():
                isalnum = True

        if isalnum:
            continue

        #print(key, value)
        emojis_copy[key] = value

    return emojis_copy

def prepare_emojis():
    global eng_emojis, chs_emojis
    # need unicode cldr annotation from
    # /usr/share/unicode/cldr/common/annotations
    eng_emojis = filter_English_emoji(load_emoji('en.xml'))
    chs_emojis = filter_Chinese_emoji(load_emoji('yue_Hans.xml'))

    eng_emojis = [(key, value) for key, value in eng_emojis.items()]
    chs_emojis = [(key, value) for key, value in chs_emojis.items()]

    compare = operator.itemgetter(0)
    eng_emojis = sorted(eng_emojis, key=compare)
    chs_emojis = sorted(chs_emojis, key=compare)


def gen_english_emojis():
    entries = []
    for match, string in eng_emojis:
        match = '"{0}"'.format(match)
        entry = '{0:<10}, "{1}"'.format(match, string)
        entries.append(entry)
    return ',\n'.join(entries)

def gen_chinese_emojis():
    entries = []
    for match, string in chs_emojis:
        match = '"{0}"'.format(match)
        entry = '{0:<10}, "{1}"'.format(match, string)
        entries.append(entry)
    return ',\n'.join(entries)


def get_table_content(tablename):
    # English Emojis
    if tablename == 'ENGLISH_EMOJIS':
        return gen_english_emojis()
    # Chinese Emojis
    if tablename == 'CHINESE_EMOJIS':
        return gen_chinese_emojis()

def expand_file(filename):
    infile = open(filename, "r")
    print(header)
    for line in infile.readlines():
        line = line.rstrip(os.linesep)
        if len(line) < 3:
            print(line)
            continue
        if line[0] == '@' and line[-1] == '@':
            tablename = line[1:-1]
            print(get_table_content(tablename))
        else:
            print(line)


### main function ###
if __name__ == "__main__":
    parser = ArgumentParser(description='Generate header file from template.')
    parser.add_argument('infile', action='store', \
                        help='input file.')

    args = parser.parse_args()
    #print(args)

    prepare_emojis()
    expand_file(args.infile)
